@@ -483,10 +483,7 @@ CreateGC(w)
 
 	if (XGetGeometry (XtDisplay(w), sbw->scrollbar.thumb, &root, &x, &y,
 		&width, &height, &bw, &depth) == 0)
-	{
-	    XtAppError (XtWidgetToApplicationContext (w),
-		 "Scrollbar Widget: Could not get geometry of thumb pixmap.");
-	}
+	    EMSG(_("Scrollbar Widget: Could not get geometry of thumb pixmap."));
     }
 
     gcValues.foreground = sbw->scrollbar.foreground;
@@ -602,7 +599,7 @@ SetValues(current, request, desired, args, num_args)
 }
 
     static void
-Resize (w)
+Resize(w)
     Widget w;
 {
     /* ForgetGravity has taken care of background, but thumb may
@@ -733,7 +730,7 @@ ExtractPosition(event, x, y, state)
     Position	    *x, *y;	/* RETURN */
     unsigned int    *state;	/* RETURN */
 {
-    switch( event->type )
+    switch (event->type)
     {
 	case MotionNotify:
 	    *x = event->xmotion.x;
@@ -779,7 +776,7 @@ HandleThumb(w, event, params, num_params)
     Position x, y, loc;
     ScrollbarWidget sbw = (ScrollbarWidget) w;
 
-    ExtractPosition(event, &x, &y, NULL);
+    ExtractPosition(event, &x, &y, (unsigned int *)NULL);
     loc = PICKLENGTH(sbw, x, y);
     /* if the motion event puts the pointer in thumb, call Move and Notify */
     /* also call Move and Notify if we're already in continuous scroll mode */
@@ -1047,7 +1044,7 @@ MoveThumb(w, event, params, num_params)
     if (!event->xmotion.same_screen)
 	return;
 
-    ExtractPosition(event, &x, &y, NULL);
+    ExtractPosition(event, &x, &y, (unsigned int *)NULL);
 
     top = FractionLoc(sbw, x, y);
 
@@ -1081,6 +1078,12 @@ NotifyThumb(w, event, params, num_params)
     Cardinal	*num_params;	/* unused */
 {
     ScrollbarWidget sbw = (ScrollbarWidget)w;
+    /* Use a union to avoid a warning for the weird conversion from float to
+     * XtPointer.  Comes from Xaw/Scrollbar.c. */
+    union {
+	XtPointer xtp;
+	float xtf;
+    } xtpf;
 
     if (LookAhead(w, event))
 	return;
@@ -1088,7 +1091,8 @@ NotifyThumb(w, event, params, num_params)
     /* thumbProc is not pretty, but is necessary for backwards
        compatibility on those architectures for which it work{s,ed};
        the intent is to pass a (truncated) float by value. */
-    XtCallCallbacks(w, XtNthumbProc, *(XtPointer*)&sbw->scrollbar.top);
+    xtpf.xtf = sbw->scrollbar.top;
+    XtCallCallbacks(w, XtNthumbProc, xtpf.xtp);
     XtCallCallbacks(w, XtNjumpProc, (XtPointer)&sbw->scrollbar.top);
 }
 
@@ -1138,7 +1142,7 @@ _Xaw3dDrawShadows(gw, event, region, out)
      *	no point to do anything if the shadow_width is 0 or the
      *	widget has not been realized.
      */
-    if ((s > 0) && XtIsRealized (gw))
+    if (s > 0 && XtIsRealized(gw))
     {
 	Dimension   h = sbw->core.height;
 	Dimension   w = sbw->core.width;
@@ -1170,7 +1174,7 @@ _Xaw3dDrawShadows(gw, event, region, out)
 	    pt[3].x = wms;  pt[3].y = s;
 	    pt[4].x =	    pt[4].y = s;
 	    pt[5].x = s;    pt[5].y = hms;
-	    XFillPolygon (dpy, win, top, pt, 6,Complex,CoordModeOrigin);
+	    XFillPolygon (dpy, win, top, pt, 6, Complex, CoordModeOrigin);
 	}
 
 	/* bottom-right shadow */
@@ -1184,7 +1188,7 @@ _Xaw3dDrawShadows(gw, event, region, out)
 	    pt[3].x = wms;  pt[3].y = s;
 	    pt[4].x = wms;  pt[4].y = hms;
 	    pt[5].x = s;    pt[5].y = hms;
-	    XFillPolygon (dpy, win, bot, pt,6, Complex,CoordModeOrigin);
+	    XFillPolygon (dpy, win, bot, pt, 6, Complex, CoordModeOrigin);
 	}
     }
 }
@@ -1212,5 +1216,5 @@ vim_XawScrollbarSetThumb(w, top, shown, max)
     sbw->scrollbar.shown = (shown > 1.0) ? 1.0 :
 		(shown >= 0.0) ? shown : sbw->scrollbar.shown;
 
-    PaintThumb (sbw);
+    PaintThumb(sbw);
 }
