@@ -26,9 +26,19 @@
 #  include <sys/select.h>
 # endif
 
+# ifndef HAVE_SELECT
+#  ifdef HAVE_SYS_POLL_H
+#   include <sys/poll.h>
+#  else
+#   ifdef HAVE_POLL_H
+#    include <poll.h>
+#   endif
+#  endif
+# endif
+
 /*
- * This file provides procedures that implement the command server functionality
- * of Vim when in contact with an X11 server.
+ * This file provides procedures that implement the command server
+ * functionality of Vim when in contact with an X11 server.
  *
  * Adapted from TCL/TK's send command  in tkSend.c of the tk 3.6 distribution.
  * Adapted for use in Vim by Flemming Madsen. Protocol changed to that of tk 4
@@ -93,10 +103,14 @@ static PendingCommand *pendingCommands = NULL;
  *
  * Each command and each result takes the form of ASCII text.  For a
  * command, the text consists of a nul character followed by several
- * nul-terminated ASCII strings.  The first string consists of the
- * single letter "c" for an expression, or "k" for keystrokes.  Subsequent
- * strings have the form "option value" where the following options are
- * supported:
+ * nul-terminated ASCII strings.  The first string consists of a
+ * single letter:
+ * "c" for an expression
+ * "k" for keystrokes
+ * "r" for reply
+ * "n" for notification.
+ * Subsequent strings have the form "option value" where the following options
+ * are supported:
  *
  * -r commWindow serial
  *
@@ -110,6 +124,10 @@ static PendingCommand *pendingCommands = NULL;
  *	"Name" gives the name of the application for which the command is
  *	intended.  This option must be present.
  *
+ * -E encoding
+ *	Encoding name used for the text.  This is the 'encoding' of the
+ *	sender.  The receiver may want to do conversion to his 'encoding'.
+ *
  * -s script
  *	"Script" is the script to be executed.  This option must be
  *	present.  Taken as a series of keystrokes in a "k" command where
@@ -174,7 +192,7 @@ static Window	LookupName __ARGS((Display *dpy, char_u *name, int delete, char_u
 static int	SendInit __ARGS((Display *dpy));
 static int	DoRegisterName __ARGS((Display *dpy, char_u *name));
 static void	DeleteAnyLingerer __ARGS((Display *dpy, Window w));
-static int	GetRegProp __ARGS((Display *dpy, char_u **regPropp, long_u *numItemsp, long_u *bytesAfterp, int msg));
+static int	GetRegProp __ARGS((Display *dpy, char_u **regPropp, long_u *numItemsp, int domsg));
 static int	WaitForPend __ARGS((void *p));
 static int	WaitForReply __ARGS((void *p));
 static int	WindowValid __ARGS((Display *dpy, Window w));
@@ -189,6 +207,8 @@ static Atom	registryProperty = None;
 static Atom	vimProperty = None;
 static int	got_x_error = FALSE;
 
+static char_u	*empty_prop = (char_u *)"";	/* empty GetRegProp() result */
+
 /*
  * Associate an ASCII name with Vim.  Try real hard to get a unique one.
  * Returns FAIL or OK.
@@ -211,7 +231,7 @@ serverRegisterName(dpy, name)
 	    if (res < -1 || i >= 1000)
 	    {
 		MSG_ATTR(_("Unable to register a command server name"),
-			 hl_attr(HLF_W));
+							      hl_attr(HLF_W));
 		return FAIL;
 	    }
 	    if (p == NULL)
@@ -386,9 +406,7 @@ serverSendToVim(dpy, name, cmd,  result, server, asExpr, localLoop, silent)
 	{
 	    char_u *ret;
 
-	    ++emsg_skip;
-	    ret = eval_to_string(cmd, NULL);
-	    --emsg_skip;
+	    ret = eval_client_expr_to_string(cmd);
 	    if (result != NULL)
 	    {
 		if (ret == NULL)
@@ -431,7 +449,7 @@ serverSendToVim(dpy, name, cmd,  result, server, asExpr, localLoop, silent)
     if (w == None)
     {
 	if (!silent)
-	    EMSG2(_("E247: no registered server named \"%s\""), name);
+	    EMSG2(_(e_noserver), name);
 	return -1;
     }
     else if (loosename != NULL)
@@ -442,18 +460,29 @@ serverSendToVim(dpy, name, cmd,  result, server, asExpr, localLoop, silent)
     /*
      * Send the command to target interpreter by appending it to the
      * comm window in the communication window.
+     * Length must be computed exactly!
      */
+#ifdef FEAT_MBYTE
+    length = STRLEN(name) + STRLEN(p_enc) + STRLEN(cmd) + 14;
+#else
     length = STRLEN(name) + STRLEN(cmd) + 10;
-    property = (char_u *)alloc((unsigned) length + 30);
+#endif
+    property = (char_u *)alloc((unsigned)length + 30);
 
+#ifdef FEAT_MBYTE
+    sprintf((char *)property, "%c%c%c-n %s%c-E %s%c-s %s",
+		      0, asExpr ? 'c' : 'k', 0, name, 0, p_enc, 0, cmd);
+#else
     sprintf((char *)property, "%c%c%c-n %s%c-s %s",
 		      0, asExpr ? 'c' : 'k', 0, name, 0, cmd);
+#endif
     if (name == loosename)
 	vim_free(loosename);
     /* Add a back reference to our comm window */
     serial++;
     sprintf((char *)property + length, "%c-r %x %d",
 						0, (int_u)commWindow, serial);
+    /* Add length of what "-r %x %d" resulted in, skipping the NUL. */
     length += STRLEN(property + length + 1) + 1;
 
     res = AppendPropCarefully(dpy, w, commProperty, property, length + 1);
@@ -536,7 +565,11 @@ WindowValid(dpy, w)
 
     for (i = 0; i < numProp; i++)
 	if (plist[i] == vimProperty)
+	{
+	    XFree(plist);
 	    return TRUE;
+	}
+    XFree(plist);
     return FALSE;
 }
 
@@ -628,7 +661,7 @@ serverGetVimNames(dpy)
     char_u	*regProp;
     char_u	*entry;
     char_u	*p;
-    long_u	numItems, bytesAfter;
+    long_u	numItems;
     int_u	w;
     garray_T	ga;
 
@@ -642,7 +675,7 @@ serverGetVimNames(dpy)
     /*
      * Read the registry property.
      */
-    if (GetRegProp(dpy, &regProp, &numItems, &bytesAfter, TRUE) == FAIL)
+    if (GetRegProp(dpy, &regProp, &numItems, TRUE) == FAIL)
 	return NULL;
 
     /*
@@ -667,7 +700,9 @@ serverGetVimNames(dpy)
 		p++;
 	}
     }
-    XFree(regProp);
+    if (regProp != empty_prop)
+	XFree(regProp);
+    ga_append(&ga, NUL);
     return ga.ga_data;
 }
 
@@ -701,9 +736,8 @@ ServerReplyFind(w, op)
 		+ serverReply.ga_len;
 	    e.id = w;
 	    ga_init2(&e.strings, 1, 100);
-	    memcpy(p, &e, sizeof(e));
+	    mch_memmove(p, &e, sizeof(e));
 	    serverReply.ga_len++;
-	    serverReply.ga_room--;
 	}
     }
     else if (p != NULL && op == SROP_Delete)
@@ -711,7 +745,6 @@ ServerReplyFind(w, op)
 	ga_clear(&p->strings);
 	mch_memmove(p, p + 1, (serverReply.ga_len - i - 1) * sizeof(*p));
 	serverReply.ga_len--;
-	serverReply.ga_room++;
     }
 
     return p;
@@ -729,13 +762,13 @@ serverStrToWin(str)
 
     sscanf((char *)str, "0x%x", &id);
     if (id == None)
-	EMSG2(_("Invalid server id used: %s"), str);
+	EMSG2(_("E573: Invalid server id used: %s"), str);
 
     return (Window)id;
 }
 
 /*
- * Send a reply string to client with id "name".
+ * Send a reply string (notification) to client with id "name".
  * Return -1 if the window is invalid.
  */
     int
@@ -757,11 +790,21 @@ serverSendReply(name, str)
     if (!WindowValid(dpy, win))
 	return -1;
 
-    length = STRLEN(str) + 7;
-    if ((property = (char_u *)alloc((unsigned) length + 30)) != NULL)
+#ifdef FEAT_MBYTE
+    length = STRLEN(p_enc) + STRLEN(str) + 14;
+#else
+    length = STRLEN(str) + 10;
+#endif
+    if ((property = (char_u *)alloc((unsigned)length + 30)) != NULL)
     {
-	sprintf((char *)property, "%c%c%c-n %s%c-w %x",
-			  0, 'n', 0, str, 0, (unsigned int)commWindow);
+#ifdef FEAT_MBYTE
+	sprintf((char *)property, "%cn%c-E %s%c-n %s%c-w %x",
+			    0, 0, p_enc, 0, str, 0, (unsigned int)commWindow);
+#else
+	sprintf((char *)property, "%cn%c-n %s%c-w %x",
+			    0, 0, str, 0, (unsigned int)commWindow);
+#endif
+	/* Add length of what "%x" resulted in. */
 	length += STRLEN(property + length);
 	res = AppendPropCarefully(dpy, win, commProperty, property, length + 1);
 	vim_free(property);
@@ -804,7 +847,6 @@ serverReadReply(dpy, win, str, localLoop)
 	{
 	    s = (char_u *) p->strings.ga_data;
 	    mch_memmove(s, s + len, p->strings.ga_len - len);
-	    p->strings.ga_room += len;
 	    p->strings.ga_len -= len;
 	}
 	else
@@ -913,13 +955,13 @@ LookupName(dpy, name, delete, loose)
 {
     char_u	*regProp, *entry;
     char_u	*p;
-    long_u	numItems, bytesAfter;
+    long_u	numItems;
     int_u	returnValue;
 
     /*
      * Read the registry property.
      */
-    if (GetRegProp(dpy, &regProp, &numItems, &bytesAfter, FALSE) == FAIL)
+    if (GetRegProp(dpy, &regProp, &numItems, FALSE) == FAIL)
 	return 0;
 
     /*
@@ -976,20 +1018,21 @@ LookupName(dpy, name, delete, loose)
 	p++;
 	count = numItems - (p - regProp);
 	if (count > 0)
-	    memcpy(entry, p, count);
+	    mch_memmove(entry, p, count);
 	XChangeProperty(dpy, RootWindow(dpy, 0), registryProperty, XA_STRING,
 			8, PropModeReplace, regProp,
 			(int)(numItems - (p - entry)));
 	XSync(dpy, False);
     }
 
-    XFree(regProp);
+    if (regProp != empty_prop)
+	XFree(regProp);
     return (Window)returnValue;
 }
 
 /*
- * Delete any lingering occurences of window id.  We promise that any
- * occurences is not ours since it is not yet put into the registry (by us)
+ * Delete any lingering occurrence of window id.  We promise that any
+ * occurrence is not ours since it is not yet put into the registry (by us)
  *
  * This is necessary in the following scenario:
  * 1. There is an old windowid for an exit'ed vim in the registry
@@ -1003,13 +1046,13 @@ DeleteAnyLingerer(dpy, win)
 {
     char_u	*regProp, *entry = NULL;
     char_u	*p;
-    long_u	numItems, bytesAfter;
-    Window	wwin;
+    long_u	numItems;
+    int_u	wwin;
 
     /*
      * Read the registry property.
      */
-    if (GetRegProp(dpy, &regProp, &numItems, &bytesAfter, FALSE) == FAIL)
+    if (GetRegProp(dpy, &regProp, &numItems, FALSE) == FAIL)
 	return;
 
     /* Scan the property for the window id.  */
@@ -1017,8 +1060,8 @@ DeleteAnyLingerer(dpy, win)
     {
 	if (*p != 0)
 	{
-	    sscanf((char *)p, "%x", (int_u *)&wwin);
-	    if (wwin == win)
+	    sscanf((char *)p, "%x", &wwin);
+	    if ((Window)wwin == win)
 	    {
 		int lastHalf;
 
@@ -1029,7 +1072,7 @@ DeleteAnyLingerer(dpy, win)
 		p++;
 		lastHalf = numItems - (p - regProp);
 		if (lastHalf > 0)
-		    memcpy(entry, p, lastHalf);
+		    mch_memmove(entry, p, lastHalf);
 		numItems = (entry - regProp) + lastHalf;
 		p = entry;
 		continue;
@@ -1048,36 +1091,49 @@ DeleteAnyLingerer(dpy, win)
 	XSync(dpy, False);
     }
 
-    XFree(regProp);
+    if (regProp != empty_prop)
+	XFree(regProp);
 }
 
 /*
  * Read the registry property.  Delete it when it's formatted wrong.
+ * Return the property in "regPropp".  "empty_prop" is used when it doesn't
+ * exist yet.
  * Return OK when successful.
  */
     static int
-GetRegProp(dpy, regPropp, numItemsp, bytesAfterp, msg)
+GetRegProp(dpy, regPropp, numItemsp, domsg)
     Display	*dpy;
     char_u	**regPropp;
     long_u	*numItemsp;
-    long_u	*bytesAfterp;
-    int		msg;		/* When TRUE give error message. */
+    int		domsg;		/* When TRUE give error message. */
 {
     int		result, actualFormat;
+    long_u	bytesAfter;
     Atom	actualType;
+    XErrorHandler old_handler;
 
     *regPropp = NULL;
+    old_handler = XSetErrorHandler(x_error_check);
+    got_x_error = FALSE;
+
     result = XGetWindowProperty(dpy, RootWindow(dpy, 0), registryProperty, 0L,
 				(long)MAX_PROP_WORDS, False,
 				XA_STRING, &actualType,
-				&actualFormat, numItemsp, bytesAfterp,
+				&actualFormat, numItemsp, &bytesAfter,
 				regPropp);
 
+    XSync(dpy, FALSE);
+    (void)XSetErrorHandler(old_handler);
+    if (got_x_error)
+	return FAIL;
+
     if (actualType == None)
     {
-	if (msg)
-	    EMSG(_("E249: couldn't read VIM instance registry property"));
-	return FAIL;
+	/* No prop yet. Logically equal to the empty list */
+	*numItemsp = 0;
+	*regPropp = empty_prop;
+	return OK;
     }
 
     /* If the property is improperly formed, then delete it. */
@@ -1086,7 +1142,7 @@ GetRegProp(dpy, regPropp, numItemsp, bytesAfterp, msg)
 	if (*regPropp != NULL)
 	    XFree(*regPropp);
 	XDeleteProperty(dpy, RootWindow(dpy, 0), registryProperty);
-	if (msg)
+	if (domsg)
 	    EMSG(_("E251: VIM instance registry property is badly formed.  Deleted!"));
 	return FAIL;
     }
@@ -1094,7 +1150,7 @@ GetRegProp(dpy, regPropp, numItemsp, bytesAfterp, msg)
 }
 
 /*
- * This procedure is invoked by the varous X event loops throughout Vims when
+ * This procedure is invoked by the various X event loops throughout Vims when
  * a property changes on the communication window.  This procedure reads the
  * property and handles command requests and responses.
  */
@@ -1108,6 +1164,7 @@ serverEventProc(dpy, eventPtr)
     int		result, actualFormat, code;
     long_u	numItems, bytesAfter;
     Atom	actualType;
+    char_u	*tofree;
 
     if (eventPtr != NULL)
     {
@@ -1159,6 +1216,7 @@ serverEventProc(dpy, eventPtr)
 	    char_u	*name, *script, *serial, *end, *res;
 	    Bool	asKeys = *p == 'k';
 	    garray_T	reply;
+	    char_u	*enc;
 
 	    /*
 	     * This is an incoming command from some other application.
@@ -1171,6 +1229,7 @@ serverEventProc(dpy, eventPtr)
 	    resWindow = None;
 	    serial = (char_u *)"";
 	    script = NULL;
+	    enc = NULL;
 	    while (p - propInfo < numItems && *p == '-')
 	    {
 		switch (p[1])
@@ -1178,7 +1237,7 @@ serverEventProc(dpy, eventPtr)
 		    case 'r':
 			end = skipwhite(p + 2);
 			resWindow = 0;
-			while (isxdigit(*end))
+			while (vim_isxdigit(*end))
 			{
 			    resWindow = 16 * resWindow + (long_u)hex2nr(*end);
 			    ++end;
@@ -1199,6 +1258,10 @@ serverEventProc(dpy, eventPtr)
 			if (p[2] == ' ')
 			    script = p + 3;
 			break;
+		    case 'E':
+			if (p[2] == ' ')
+			    enc = p + 3;
+			break;
 		}
 		while (*p != 0)
 		    p++;
@@ -1215,22 +1278,26 @@ serverEventProc(dpy, eventPtr)
 	    if (resWindow != None)
 	    {
 		ga_init2(&reply, 1, 100);
+#ifdef FEAT_MBYTE
+		ga_grow(&reply, 50 + STRLEN(p_enc));
+		sprintf(reply.ga_data, "%cr%c-E %s%c-s %s%c-r ",
+						   0, 0, p_enc, 0, serial, 0);
+		reply.ga_len = 14 + STRLEN(p_enc) + STRLEN(serial);
+#else
 		ga_grow(&reply, 50);
 		sprintf(reply.ga_data, "%cr%c-s %s%c-r ", 0, 0, serial, 0);
 		reply.ga_len = 10 + STRLEN(serial);
-		reply.ga_room -= reply.ga_len;
+#endif
 	    }
 	    res = NULL;
 	    if (serverName != NULL && STRICMP(name, serverName) == 0)
 	    {
+		script = serverConvert(enc, script, &tofree);
 		if (asKeys)
 		    server_to_input_buf(script);
 		else
-		{
-		    ++emsg_skip;
-		    res = eval_to_string(script, NULL);
-		    --emsg_skip;
-		}
+		    res = eval_client_expr_to_string(script);
+		vim_free(tofree);
 	    }
 	    if (resWindow != None)
 	    {
@@ -1242,17 +1309,19 @@ serverEventProc(dpy, eventPtr)
 		    ga_append(&reply, 0);
 		    ga_concat(&reply, (char_u *)"-c 1");
 		}
-		ga_append(&reply, 0);
+		ga_append(&reply, NUL);
 		(void)AppendPropCarefully(dpy, resWindow, commProperty,
 					   reply.ga_data, reply.ga_len);
+		ga_clear(&reply);
 	    }
 	    vim_free(res);
 	}
 	else if (*p == 'r' && p[1] == 0)
 	{
-	    int	    serial, gotSerial;
-	    char_u  *res;
-	    PendingCommand *pcPtr;
+	    int		    serial, gotSerial;
+	    char_u	    *res;
+	    PendingCommand  *pcPtr;
+	    char_u	    *enc;
 
 	    /*
 	     * This is a reply to some command that we sent out.  Iterate
@@ -1263,6 +1332,7 @@ serverEventProc(dpy, eventPtr)
 	    gotSerial = 0;
 	    res = (char_u *)"";
 	    code = 0;
+	    enc = NULL;
 	    while ((p-propInfo) < numItems && *p == '-')
 	    {
 		switch (p[1])
@@ -1271,6 +1341,10 @@ serverEventProc(dpy, eventPtr)
 			if (p[2] == ' ')
 			    res = p + 3;
 			break;
+		    case 'E':
+			if (p[2] == ' ')
+			    enc = p + 3;
+			break;
 		    case 's':
 			if (sscanf((char *)p + 2, " %d", &serial) == 1)
 			    gotSerial = 1;
@@ -1299,7 +1373,12 @@ serverEventProc(dpy, eventPtr)
 
 		pcPtr->code = code;
 		if (res != NULL)
-		    pcPtr->result = vim_strsave(res);
+		{
+		    res = serverConvert(enc, res, &tofree);
+		    if (tofree == NULL)
+			res = vim_strsave(res);
+		    pcPtr->result = res;
+		}
 		else
 		    pcPtr->result = vim_strsave((char_u *)"");
 		break;
@@ -1311,8 +1390,8 @@ serverEventProc(dpy, eventPtr)
 	    unsigned int u;
 	    int		gotWindow;
 	    char_u	*str;
-	    char_u	winstr[30];
 	    struct	ServerReply *r;
+	    char_u	*enc;
 
 	    /*
 	     * This is a (n)otification.  Sent with serverreply_send in VimL.
@@ -1321,6 +1400,7 @@ serverEventProc(dpy, eventPtr)
 	    p += 2;
 	    gotWindow = 0;
 	    str = (char_u *)"";
+	    enc = NULL;
 	    while ((p-propInfo) < numItems && *p == '-')
 	    {
 		switch (p[1])
@@ -1329,6 +1409,10 @@ serverEventProc(dpy, eventPtr)
 			if (p[2] == ' ')
 			    str = p + 3;
 			break;
+		    case 'E':
+			if (p[2] == ' ')
+			    enc = p + 3;
+			break;
 		    case 'w':
 			if (sscanf((char *)p + 2, " %x", &u) == 1)
 			{
@@ -1344,16 +1428,21 @@ serverEventProc(dpy, eventPtr)
 
 	    if (!gotWindow)
 		continue;
+	    str = serverConvert(enc, str, &tofree);
 	    if ((r = ServerReplyFind(win, SROP_Add)) != NULL)
 	    {
 		ga_concat(&(r->strings), str);
-		ga_append(&(r->strings), 0);
+		ga_append(&(r->strings), NUL);
 	    }
 #ifdef FEAT_AUTOCMD
-	    sprintf((char *)winstr, "0x%x", (unsigned int)win);
-	    apply_autocmds(EVENT_REMOTEREPLY, winstr, str, TRUE, curbuf);
-#endif
+	    {
+		char_u	winstr[30];
 
+		sprintf((char *)winstr, "0x%x", (unsigned int)win);
+		apply_autocmds(EVENT_REMOTEREPLY, winstr, str, TRUE, curbuf);
+	    }
+#endif
+	    vim_free(tofree);
 	}
 	else
 	{
@@ -1420,6 +1509,6 @@ IsSerialName(str)
 {
     int len = STRLEN(str);
 
-    return (len > 1 && isdigit(str[len - 1]));
+    return (len > 1 && vim_isdigit(str[len - 1]));
 }
 #endif	/* FEAT_CLIENTSERVER */
