@@ -774,7 +774,7 @@ restofline:
 		    *namebuf = NUL;
 		    if (tail && *tail)
 		    {
-			STRCPY(IObuff, skipwhite(tail));
+			STRMOVE(IObuff, skipwhite(tail));
 			multiscan = TRUE;
 			goto restofline;
 		    }
@@ -1415,10 +1415,12 @@ qf_jump(qi, dir, errornr, forceit)
     char_u		*line;
 #ifdef FEAT_WINDOWS
     char_u		*old_swb = p_swb;
+    unsigned		old_swb_flags = swb_flags;
     int			opened_window = FALSE;
     win_T		*win;
     win_T		*altwin;
 #endif
+    win_T		*oldwin = curwin;
     int			print_message = TRUE;
     int			len;
 #ifdef FEAT_FOLDING
@@ -1594,10 +1596,10 @@ qf_jump(qi, dir, errornr, forceit)
 	    }
 
 	/*
-	 * If no usable window is found and 'switchbuf' is set to 'usetab'
+	 * If no usable window is found and 'switchbuf' contains "usetab"
 	 * then search in other tabs.
 	 */
-	if (!usable_win && vim_strchr(p_swb, 'a') != NULL)
+	if (!usable_win && (swb_flags & SWB_USETAB))
 	{
 	    tabpage_T	*tp;
 	    win_T	*wp;
@@ -1625,6 +1627,7 @@ qf_jump(qi, dir, errornr, forceit)
 		goto failed;		/* not enough room for window */
 	    opened_window = TRUE;	/* close it when fail */
 	    p_swb = empty_option;	/* don't split again */
+	    swb_flags = 0;
 # ifdef FEAT_SCROLLBIND
 	    curwin->w_p_scb = FALSE;
 # endif
@@ -1742,7 +1745,8 @@ qf_jump(qi, dir, errornr, forceit)
 	    }
 	    else
 		ok = do_ecmd(qf_ptr->qf_fnum, NULL, NULL, NULL, (linenr_T)1,
-						   ECMD_HIDE + ECMD_SET_HELP);
+					   ECMD_HIDE + ECMD_SET_HELP,
+					   oldwin == curwin ? curwin : NULL);
 	}
 	else
 	    ok = buflist_getfile(qf_ptr->qf_fnum,
@@ -1866,7 +1870,10 @@ theend:
 	/* Restore old 'switchbuf' value, but not when an autocommand or
 	 * modeline has changed the value. */
 	if (p_swb == empty_option)
+	{
 	    p_swb = old_swb;
+	    swb_flags = old_swb_flags;
+	}
 	else
 	    free_string_option(old_swb);
     }
@@ -2262,6 +2269,7 @@ ex_copen(eap)
     win_T	*win;
     tabpage_T	*prevtab = curtab;
     buf_T	*qf_buf;
+    win_T	*oldwin = curwin;
 
     if (eap->cmdidx == CMD_lopen || eap->cmdidx == CMD_lwindow)
     {
@@ -2321,14 +2329,16 @@ ex_copen(eap)
 	    win->w_llist->qf_refcount++;
 	}
 
+	if (oldwin != curwin)
+	    oldwin = NULL;  /* don't store info when in another window */
 	if (qf_buf != NULL)
 	    /* Use the existing quickfix buffer */
 	    (void)do_ecmd(qf_buf->b_fnum, NULL, NULL, NULL, ECMD_ONE,
-						     ECMD_HIDE + ECMD_OLDBUF);
+					     ECMD_HIDE + ECMD_OLDBUF, oldwin);
 	else
 	{
 	    /* Create a new quickfix buffer */
-	    (void)do_ecmd(0, NULL, NULL, NULL, ECMD_ONE, ECMD_HIDE);
+	    (void)do_ecmd(0, NULL, NULL, NULL, ECMD_ONE, ECMD_HIDE, oldwin);
 	    /* switch off 'swapfile' */
 	    set_option_value((char_u *)"swf", 0L, NULL, OPT_LOCAL);
 	    set_option_value((char_u *)"bt", 0L, (char_u *)"quickfix",
@@ -2994,14 +3004,14 @@ ex_vimgrep(eap)
     char_u	*save_ei = NULL;
 #endif
     aco_save_T	aco;
-#ifdef FEAT_AUTOCMD
-    char_u	*au_name =  NULL;
     int		flags = 0;
     colnr_T	col;
     long	tomatch;
     char_u	dirname_start[MAXPATHL];
     char_u	dirname_now[MAXPATHL];
     char_u	*target_dir = NULL;
+#ifdef FEAT_AUTOCMD
+    char_u	*au_name =  NULL;
 
     switch (eap->cmdidx)
     {
@@ -3774,7 +3784,7 @@ ex_helpgrep(eap)
 
     /* Make 'cpoptions' empty, the 'l' flag should not be used here. */
     save_cpo = p_cpo;
-    p_cpo = (char_u *)"";
+    p_cpo = empty_option;
 
 #ifdef FEAT_MULTI_LANG
     /* Check for a specified language */
@@ -3884,7 +3894,11 @@ ex_helpgrep(eap)
 	qi->qf_lists[qi->qf_curlist].qf_index = 1;
     }
 
-    p_cpo = save_cpo;
+    if (p_cpo == empty_option)
+	p_cpo = save_cpo;
+    else
+	/* Darn, some plugin changed the value. */
+	free_string_option(save_cpo);
 
 #ifdef FEAT_WINDOWS
     qf_update_buffer(qi);
