@@ -11,16 +11,12 @@
  * Code to handle tags and the tag stack
  */
 
-#if defined MSDOS || defined WIN32 || defined(_WIN64)
+#if defined(MSDOS) || defined(WIN16) || defined(WIN32) || defined(_WIN64)
 # include "vimio.h"	/* for lseek(), must be before vim.h */
 #endif
 
 #include "vim.h"
 
-#ifdef HAVE_FCNTL_H
-# include <fcntl.h>	/* for lseek() */
-#endif
-
 /*
  * Structure to hold pointers to various items in a tag line.
  */
@@ -2664,7 +2660,7 @@ get_tagfname(tnp, first, buf)
 	    /* move the filename one char forward and truncate the
 	     * filepath with a NUL */
 	    filename = gettail(buf);
-	    mch_memmove(filename + 1, filename, STRLEN(filename) + 1);
+	    STRMOVE(filename + 1, filename);
 	    *filename++ = NUL;
 
 	    tnp->tn_search_ctx = vim_findfile_init(buf, filename,
@@ -2729,7 +2725,24 @@ parse_tag_line(lbuf,
 	 */
 	p_7f = vim_strchr(lbuf, 0x7f);
 	if (p_7f == NULL)
+	{
+etag_fail:
+	    if (vim_strchr(lbuf, '\n') == NULL)
+	    {
+		/* Truncated line.  Ignore it. */
+		if (p_verbose >= 5)
+		{
+		    verbose_enter();
+		    MSG(_("Ignoring long line in tags file"));
+		    verbose_leave();
+		}
+		tagp->command = lbuf;
+		tagp->tagname = lbuf;
+		tagp->tagname_end = lbuf;
+		return OK;
+	    }
 	    return FAIL;
+	}
 
 	/* Find ^A.  If not found the line number is after the 0x7f */
 	p = vim_strchr(p_7f, Ctrl_A);
@@ -2739,7 +2752,7 @@ parse_tag_line(lbuf,
 	    ++p;
 
 	if (!VIM_ISDIGIT(*p))	    /* check for start of line number */
-	    return FAIL;
+	    goto etag_fail;
 	tagp->command = p;
 
 
@@ -2753,7 +2766,7 @@ parse_tag_line(lbuf,
 	    /* find end of tagname */
 	    for (p = p_7f - 1; !vim_iswordc(*p); --p)
 		if (p == lbuf)
-		    return FAIL;
+		    goto etag_fail;
 	    tagp->tagname_end = p + 1;
 	    while (p >= lbuf && vim_iswordc(*p))
 		--p;
@@ -3391,13 +3404,6 @@ expand_tag_fname(fname, tag_fname, expand)
     return retval;
 }
 
-/*
- * Moves the tail part of the path (including the terminating NUL) pointed to
- * by "tail" to the new location pointed to by "here". This should accomodate
- * an overlapping move.
- */
-#define movetail(here, tail)  mch_memmove(here, tail, STRLEN(tail) + (size_t)1)
-
 /*
  * Converts a file name into a canonical form. It simplifies a file name into
  * its simplest form by stripping out unneeded components, if any.  The
@@ -3451,7 +3457,7 @@ simplify_filename(filename)
 	else
 #endif
 	  if (vim_ispathsep(*p))
-	    movetail(p, p + 1);		/* remove duplicate "/" */
+	    STRMOVE(p, p + 1);		/* remove duplicate "/" */
 	else if (p[0] == '.' && (vim_ispathsep(p[1]) || p[1] == NUL))
 	{
 	    if (p == start && relative)
@@ -3468,7 +3474,7 @@ simplify_filename(filename)
 			mb_ptr_adv(tail);
 		else if (p > start)
 		    --p;		/* strip preceding path separator */
-		movetail(p, tail);
+		STRMOVE(p, tail);
 	    }
 	}
 	else if (p[0] == '.' && p[1] == '.' &&
@@ -3584,19 +3590,19 @@ simplify_filename(filename)
 		    {
 			if (p > start && tail[-1] == '.')
 			    --p;
-			movetail(p, tail);	/* strip previous component */
+			STRMOVE(p, tail);	/* strip previous component */
 		    }
 
 		    --components;
 		}
 	    }
 	    else if (p == start && !relative)	/* leading "/.." or "/../" */
-		movetail(p, tail);		/* strip ".." or "../" */
+		STRMOVE(p, tail);		/* strip ".." or "../" */
 	    else
 	    {
 		if (p == start + 2 && p[-2] == '.')	/* leading "./../" */
 		{
-		    movetail(p - 2, p);			/* strip leading "./" */
+		    STRMOVE(p - 2, p);			/* strip leading "./" */
 		    tail -= 2;
 		}
 		p = tail;		/* skip to char after ".." or "../" */
@@ -3865,6 +3871,8 @@ get_tags(list, pat)
 			    /* Skip field without colon. */
 			    while (*p != NUL && *p >= ' ')
 				++p;
+			if (*p == NUL)
+			    break;
 		    }
 		}
 	    }
