@@ -217,6 +217,11 @@ static char *(features[]) =
 #else
 	"-find_in_path",
 #endif
+#ifdef FEAT_FLOAT
+	"+float",
+#else
+	"-float",
+#endif
 #ifdef FEAT_FOLDING
 	"+folding",
 #else
@@ -345,6 +350,11 @@ static char *(features[]) =
 # else
 	"-mouse_netterm",
 # endif
+# ifdef FEAT_SYSMOUSE
+	"+mouse_sysmouse",
+# else
+	"-mouse_sysmouse",
+# endif
 # ifdef FEAT_MOUSE_XTERM
 	"+mouse_xterm",
 # else
@@ -666,476 +676,6 @@ static char *(features[]) =
 
 static int included_patches[] =
 {   /* Add new patch number below this line */
-/**/
-    314,
-/**/
-    313,
-/**/
-    312,
-/**/
-    311,
-/**/
-    310,
-/**/
-    309,
-/**/
-    308,
-/**/
-    307,
-/**/
-    306,
-/**/
-    305,
-/**/
-    304,
-/**/
-    303,
-/**/
-    302,
-/**/
-    301,
-/**/
-    300,
-/**/
-    299,
-/**/
-    298,
-/**/
-    297,
-/**/
-    296,
-/**/
-    295,
-/**/
-    294,
-/**/
-    293,
-/**/
-    292,
-/**/
-    291,
-/**/
-    290,
-/**/
-    289,
-/**/
-    288,
-/**/
-    287,
-/**/
-    286,
-/**/
-    285,
-/**/
-    284,
-/**/
-    283,
-/**/
-    282,
-/**/
-    281,
-/**/
-    280,
-/**/
-    279,
-/**/
-    278,
-/**/
-    277,
-/**/
-    276,
-/**/
-    275,
-/**/
-    274,
-/**/
-    273,
-/**/
-    272,
-/**/
-    271,
-/**/
-    270,
-/**/
-    269,
-/**/
-    268,
-/**/
-    267,
-/**/
-    266,
-/**/
-    265,
-/**/
-    264,
-/**/
-    263,
-/**/
-    262,
-/**/
-    261,
-/**/
-    260,
-/**/
-    259,
-/**/
-    258,
-/**/
-    257,
-/**/
-    256,
-/**/
-    255,
-/**/
-    254,
-/**/
-    253,
-/**/
-    252,
-/**/
-    251,
-/**/
-    250,
-/**/
-    249,
-/**/
-    248,
-/**/
-    247,
-/**/
-    246,
-/**/
-    245,
-/**/
-    244,
-/**/
-    243,
-/**/
-    242,
-/**/
-    241,
-/**/
-    240,
-/**/
-    239,
-/**/
-    238,
-/**/
-    237,
-/**/
-    236,
-/**/
-    235,
-/**/
-    234,
-/**/
-    233,
-/**/
-    232,
-/**/
-    231,
-/**/
-    230,
-/**/
-    229,
-/**/
-    228,
-/**/
-    227,
-/**/
-    226,
-/**/
-    225,
-/**/
-    224,
-/**/
-    223,
-/**/
-    222,
-/**/
-    221,
-/**/
-    220,
-/**/
-    219,
-/**/
-    218,
-/**/
-    217,
-/**/
-    216,
-/**/
-    215,
-/**/
-    214,
-/**/
-    213,
-/**/
-    212,
-/**/
-    211,
-/**/
-    210,
-/**/
-    209,
-/**/
-    208,
-/**/
-    207,
-/**/
-    206,
-/**/
-    205,
-/**/
-    204,
-/**/
-    203,
-/**/
-    202,
-/**/
-    201,
-/**/
-    200,
-/**/
-    199,
-/**/
-    198,
-/**/
-    197,
-/**/
-    196,
-/**/
-    195,
-/**/
-    194,
-/**/
-    193,
-/**/
-    192,
-/**/
-    191,
-/**/
-    190,
-/**/
-    189,
-/**/
-    188,
-/**/
-    187,
-/**/
-    186,
-/**/
-    185,
-/**/
-    184,
-/**/
-    183,
-/**/
-    182,
-/**/
-    181,
-/**/
-    180,
-/**/
-    179,
-/**/
-    178,
-/**/
-    177,
-/**/
-    176,
-/**/
-    175,
-/**/
-    174,
-/**/
-    173,
-/**/
-    172,
-/**/
-    171,
-/**/
-    170,
-/**/
-    169,
-/**/
-    168,
-/**/
-    167,
-/**/
-    166,
-/**/
-    165,
-/**/
-    164,
-/**/
-    163,
-/**/
-    162,
-/**/
-    161,
-/**/
-    160,
-/**/
-    159,
-/**/
-    158,
-/**/
-    157,
-/**/
-    156,
-/**/
-    155,
-/**/
-    154,
-/**/
-    153,
-/**/
-    152,
-/**/
-    151,
-/**/
-    150,
-/**/
-    149,
-/**/
-    148,
-/**/
-    147,
-/**/
-    146,
-/**/
-    145,
-/**/
-    144,
-/**/
-    143,
-/**/
-    142,
-/**/
-    141,
-/**/
-    140,
-/**/
-    139,
-/**/
-    138,
-/**/
-    137,
-/**/
-    136,
-/**/
-    135,
-/**/
-    134,
-/**/
-    133,
-/**/
-    132,
-/**/
-    131,
-/**/
-    130,
-/**/
-    129,
-/**/
-    128,
-/**/
-    127,
-/**/
-    126,
-/**/
-    125,
-/**/
-    124,
-/**/
-    123,
-/**/
-    122,
-/**/
-    121,
-/**/
-    120,
-/**/
-    119,
-/**/
-    118,
-/**/
-    117,
-/**/
-    116,
-/**/
-    115,
-/**/
-    114,
-/**/
-    113,
-/**/
-    112,
-/**/
-    111,
-/**/
-    110,
-/**/
-    109,
-/**/
-    108,
-/**/
-    107,
-/**/
-    106,
-/**/
-    105,
-/**/
-    104,
-/**/
-    103,
-/**/
-    102,
-/**/
-    101,
-/**/
-    100,
-/**/
-    99,
-/**/
-    98,
-/**/
-    97,
-/**/
-    96,
-/**/
-    95,
-/**/
-    94,
-/**/
-    93,
-/**/
-    92,
-/**/
-    91,
-/**/
-    90,
-/**/
-    89,
-/**/
-    88,
-/**/
-    87,
-/**/
-    86,
-/**/
-    85,
-/**/
-    84,
-/**/
-    83,
-/**/
-    82,
-/**/
-    81,
-/**/
-    80,
 /**/
     79,
 /**/
@@ -1298,6 +838,19 @@ static int included_patches[] =
     0
 };
 
+/*
+ * Place to put a short description when adding a feature with a patch.
+ * Keep it short, e.g.,: "relative numbers", "persistent undo".
+ * Also add a comment marker to separate the lines.
+ * See the official Vim patches for the diff format: It must use a context of
+ * one line only.  Create it by hand or use "diff -C2" and edit the patch.
+ */
+static char *(extra_patches[]) =
+{   /* Add your patch description below this line */
+/**/
+    NULL
+};
+
     int
 highest_patch()
 {
@@ -1357,12 +910,12 @@ list_version()
 # ifdef FEAT_GUI_W32
 #  if defined(_MSC_VER) && (_MSC_VER <= 1010)
     /* Only MS VC 4.1 and earlier can do Win32s */
-    MSG_PUTS(_("\nMS-Windows 16/32 bit GUI version"));
+    MSG_PUTS(_("\nMS-Windows 16/32-bit GUI version"));
 #  else
 #   ifdef _WIN64
-    MSG_PUTS(_("\nMS-Windows 64 bit GUI version"));
+    MSG_PUTS(_("\nMS-Windows 64-bit GUI version"));
 #   else
-    MSG_PUTS(_("\nMS-Windows 32 bit GUI version"));
+    MSG_PUTS(_("\nMS-Windows 32-bit GUI version"));
 #   endif
 #  endif
     if (gui_is_win32s())
@@ -1371,17 +924,21 @@ list_version()
     MSG_PUTS(_(" with OLE support"));
 # endif
 # else
-    MSG_PUTS(_("\nMS-Windows 32 bit console version"));
+#  ifdef _WIN64
+    MSG_PUTS(_("\nMS-Windows 64-bit console version"));
+#  else
+    MSG_PUTS(_("\nMS-Windows 32-bit console version"));
+#  endif
 # endif
 #endif
 #ifdef WIN16
-    MSG_PUTS(_("\nMS-Windows 16 bit version"));
+    MSG_PUTS(_("\nMS-Windows 16-bit version"));
 #endif
 #ifdef MSDOS
 # ifdef DJGPP
-    MSG_PUTS(_("\n32 bit MS-DOS version"));
+    MSG_PUTS(_("\n32-bit MS-DOS version"));
 # else
-    MSG_PUTS(_("\n16 bit MS-DOS version"));
+    MSG_PUTS(_("\n16-bit MS-DOS version"));
 # endif
 #endif
 #ifdef MACOS
@@ -1400,7 +957,7 @@ list_version()
     MSG_PUTS(_("\nRISC OS version"));
 #endif
 #ifdef VMS
-    MSG_PUTS("\nOpenVMS version");
+    MSG_PUTS(_("\nOpenVMS version"));
 # ifdef HAVE_PATHDEF
     if (*compiled_arch != NUL)
     {
@@ -1439,6 +996,19 @@ list_version()
 	}
     }
 
+    /* Print the list of extra patch descriptions if there is at least one. */
+    if (extra_patches[0] != NULL)
+    {
+	MSG_PUTS(_("\nExtra patches: "));
+	s = "";
+	for (i = 0; extra_patches[i] != NULL; ++i)
+	{
+	    MSG_PUTS(s);
+	    s = ", ";
+	    MSG_PUTS(extra_patches[i]);
+	}
+    }
+
 #ifdef MODIFIED_BY
     MSG_PUTS("\n");
     MSG_PUTS(_("Modified by "));
